/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: CMember.java,v 1.2 2002/03/08 21:54:18 mrobilla Exp $
 */

package at.dms.kjc;

/**
 * This class represents an exported member of a class
 * @see	CMember
 * @see	CField
 * @see	CMethod
 * @see	CClass
 */
public class CMember extends at.dms.util.Utils implements Constants {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * Constructs a field export
   * @param	owner		the owner of this member
   * @param	modifiers	the modifiers on this member
   * @param	ident		the ident of this member
   * @param	deprecated	is this member deprecated
   */
  public CMember(CClass owner,
		 int modifiers,
		 String ident,
		 boolean deprecated)
  {
    this.owner = owner;
    this.modifiers = modifiers;
    this.ident = ident;
    this.deprecated = deprecated;
  }

  // ----------------------------------------------------------------------
  // ACCESSORS
  // ----------------------------------------------------------------------

  /**
   * Returns the owner of this member
   */
  public CClass getOwner() {
    return owner;
  }

  /**
   * @return the ident of this method
   */
  public String getIdent() {
    return ident;
  }

  /**
   * @return the fully qualified name of this member
   */
  public String getQualifiedName() {
    return owner.getQualifiedName() + JAV_NAME_SEPARATOR + ident;
  }

  /**
   * @return the fully qualified name of this member
   */
  public String getPrefixName() {
    return owner.getQualifiedName();
  }

  /**
   * @return the fully qualified name of this member
   */
  public String getJavaName() {
    return getQualifiedName().replace('/' , '.');
  }

  /**
   * @return the modifiers of this field
   */
  public int getModifiers() {
    return modifiers;
  }

  // ----------------------------------------------------------------------
  // ACCESSORS (QUICK)
  // ----------------------------------------------------------------------

  /**
   * @return	the interface
   */
  public CField getField() {
    fail();
    return null;
  }

  /**
   * @return	the interface
   */
  public CMethod getMethod() {
    fail();
    return null;
  }

  /**
   * @return	the interface
   */
  public CClass getCClass() {
    fail();
    return null;
  }

  // ----------------------------------------------------------------------
  // ACCESSORS (QUICK)
  // ----------------------------------------------------------------------

  /**
   * @return	true if this member is static
   */
  public boolean isStatic() {
    return hasFlag(modifiers, ACC_STATIC);
  }

  /**
   * @return	true if this member is public
   */
  public boolean isPublic() {
    return hasFlag(modifiers, ACC_PUBLIC);
  }

  /**
   * @return	true if this member is protected
   */
  public boolean isProtected() {
    return hasFlag(modifiers, ACC_PROTECTED);
  }

  /**
   * @return	true if this member is private
   */
  public boolean isPrivate() {
    return hasFlag(modifiers, ACC_PRIVATE);
  }

  /**
   * @return	true if this member is final
   */
  public boolean isFinal() {
    return hasFlag(modifiers, ACC_FINAL);
  }

  /**
   * @return	true if this member is deprecated
   */
  public boolean isDeprecated() {
    return deprecated;
  }

  /**
   * @return	true if this member is accessible
   */
  public boolean isAccessible(CClass from) {
    if (isPublic() || from.isDefinedInside(owner) || owner.isDefinedInside(from)) {
      return true;
    }

    if (from.isInnerClass()) {
      if (isAccessible(from.getOwner())) {
	return true;
      }
    }

    if (!isPrivate() && owner.getPackage() == from.getPackage()) {
      return true;
    }

    if (isProtected() && from.descendsFrom(owner)) {
      return true;
    }

    return false;
  }

  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  protected CClass			owner;	//!!! make private
  private int				modifiers;
  private String			ident;
  private boolean			deprecated;
}
