/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: CMethodContext.java,v 1.2 2002/03/08 21:54:18 mrobilla Exp $
 */

package at.dms.kjc;

import java.util.Hashtable;
import java.util.Enumeration;

import at.dms.compiler.CWarning;
import at.dms.compiler.TokenReference;
import at.dms.compiler.PositionedError;

/**
 * This class represents a method context during check
 * @see CContext
 * @see CCompilationUnitContext
 * @see CClassContext
 * @see CMethodContext
 * @see CConstructorContext
 * @see CInitializerContext
 * @see CBodyContext
 * @see CBlockContext
 */
public class CMethodContext extends CContext {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * CMethodContext
   * @param	parent		the parent context
   * @param	self		the corresponding method interface
   */
  CMethodContext(CClassContext parent, CMethod self) {
    super(parent);
    this.self = self;
  }

  /**
   * Verify that all checked exceptions are defined in the throw list
   * and return types are valid
   * @exception	UnpositionedError	this error will be positioned soon
   */
  public void close(TokenReference ref) throws PositionedError {
    Enumeration		enum = throwables.elements();
    CClassType[]	checked = self.getThrowables();
    boolean[]		used = new boolean[checked.length];
    
  loop:
    while (enum.hasMoreElements()) {
      CThrowableInfo	thrown = (CThrowableInfo)enum.nextElement();
      CClassType	type = thrown.getThrowable();
      
      // only checked exceptions need to be checked
      if (! type.isCheckedException()) {
	continue loop;
      }
      
      for (int j = 0; j < checked.length; j++) {
	if (type.isAssignableTo(checked[j])) {
	  used[j] = true;
	  continue loop;
	}
      }
      throw new PositionedError(thrown.getLocation().getTokenReference(),
				KjcMessages.METHOD_UNCATCHED_EXCEPTION,
				type,
				null);
    }
    
    for (int i = 0; i < checked.length; i++) {
      if (!checked[i].isCheckedException()) {
	reportTrouble(new CWarning(ref,
				   KjcMessages.METHOD_UNCHECKED_EXCEPTION,
				   checked[i],
				   null));
      } else if (!used[i]) {
	reportTrouble(new CWarning(ref,
				   KjcMessages.METHOD_UNTHROWN_EXCEPTION,
				   checked[i],
				   null));
      }
    }
  }

  // ----------------------------------------------------------------------
  // ACCESSORS (NEW CONTEXT)
  // ----------------------------------------------------------------------

  public CBlockContext createBlockContext(int localVars) {
    return new CBlockContext(this, localVars);
  }

  // ----------------------------------------------------------------------
  // ACCESSORS
  // ----------------------------------------------------------------------

  /**
   * getClassContext
   * @return	the near parent of type CClassContext
   */
  public CClassContext getClassContext() {
    return getParentContext().getClassContext();
  }

  /**
   * getCMethod
   * @return	the near parent of type CMethodContext
   */
  public CMethod getCMethod() {
    return self;
  }

  /**
   * getMethod
   * @return	the near parent of type CMethodContext
   */
  public CMethodContext getMethodContext() {
    return this;
  }

  public int localsPosition() {
    return 0;
  }

  // ----------------------------------------------------------------------
  // THROWABLES
  // ----------------------------------------------------------------------

  /**
   * @param	throwable	the type of the new throwable
   */
  public void addThrowable(CThrowableInfo throwable) {
    throwables.put(throwable.getThrowable().toString(), throwable);
  }

  /**
   * @return the list of exception that may be thrown
   */
  public Hashtable getThrowables() {
    return throwables;
  }

  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  protected	Hashtable	throwables = new Hashtable();
  protected	Hashtable	labels;		// Hashtable<String, String>
  private	CMethod		self;
}
