/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: CParseCompilationUnitContext.java,v 1.2 2002/03/08 21:54:19 mrobilla Exp $
 */

package at.dms.kjc;

import java.util.Vector;
import java.util.Stack;
import at.dms.compiler.Compiler;

public class CParseCompilationUnitContext {
  public static CParseCompilationUnitContext getInstance() {
    return stack.size() == 0 ?
      new CParseCompilationUnitContext() :
      (CParseCompilationUnitContext)stack.pop();
  }

  public void release() {
    release(this);
  }

  public static void release(CParseCompilationUnitContext context) {
    context.clear();
    stack.push(context);
  }

  private CParseCompilationUnitContext() {
    // 7.5.3 Automatic Imports
    addPackageImport(JAVA_LANG);
  }

  private void clear() {
    packageImports.setSize(0);
    classImports.setSize(0);
    typeDeclarations.setSize(0);

    // 7.5.3 Automatic Imports
    addPackageImport(JAVA_LANG);
    pack = null;
  }

  // ----------------------------------------------------------------------
  // ACCESSORS (ADD)
  // ----------------------------------------------------------------------

  public void setPackage(JPackageName pack) {
    packageName = pack == JPackageName.UNNAMED ? "" : pack.getName() + '/';
    this.pack = pack;
  }

  public void addPackageImport(JPackageImport pack) {
    packageImports.addElement(pack);
  }

  public void addClassImport(JClassImport clazz) {
    classImports.addElement(clazz);
  }

  public void addTypeDeclaration(Compiler compiler, JTypeDeclaration decl) {
    typeDeclarations.addElement(decl);
    decl.generateInterface(compiler, null, packageName);
  }

  // ----------------------------------------------------------------------
  // ACCESSORS (GET)
  // ----------------------------------------------------------------------

  public JPackageImport[] getPackageImports() {
    JPackageImport[]	temp = new JPackageImport[packageImports.size()];

    packageImports.copyInto(temp);

    return temp;
  }

  public JClassImport[] getClassImports() {
    JClassImport[]	temp = new JClassImport[classImports.size()];

    classImports.copyInto(temp);

    return temp;    
  }

  public JTypeDeclaration[] getTypeDeclarations() {
    JTypeDeclaration[]	temp = new JTypeDeclaration[typeDeclarations.size()];

    typeDeclarations.copyInto(temp);

    return temp;    
  }

  public JPackageName getPackageName() {
    return pack;
  }

  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  public static final JPackageImport	JAVA_LANG =
    new JPackageImport(at.dms.compiler.TokenReference.NO_REF, Constants.JAV_RUNTIME, null);

  private JPackageName pack;
  private String packageName;
  private Vector packageImports = new Vector();
  private Vector classImports = new Vector();
  private Vector typeDeclarations = new Vector();

  private static Stack stack = new Stack();
}
