/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: CSourceClass.java,v 1.2 2002/03/08 21:54:19 mrobilla Exp $
 */

package at.dms.kjc;

import java.io.IOException;
import java.util.Enumeration;
import java.util.Hashtable;

import at.dms.classfile.ClassFileFormatException;
import at.dms.classfile.FieldRefInstruction;
import at.dms.classfile.FieldInfo;
import at.dms.compiler.Compiler;
import at.dms.compiler.TokenReference;

/**
 * This class represents the exported members of a class (inner classes, methods and fields)
 * It is build from a parsed files so values are accessibles differently after build and
 * after interface checked
 */
public class CSourceClass extends CClass {
  //$$$ OPTIMIZE OUTERS HANDLING

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * Constructs a class export from source
   */
  public CSourceClass(Compiler compiler,
		      CClass owner,
		      TokenReference where,
		      int modifiers,
		      String ident,
		      String qualifiedName,
		      boolean deprecated)
  {
    super(compiler, owner, where.getFile(), modifiers, ident, qualifiedName, null, deprecated);
  }

  /**
   * Ends the definition of this class
   */
  public void close(CClassType[] interfaces,
		    CClassType superClass,
		    Hashtable fields_H,
		    CMethod[] methods,
		    Hashtable methods_H) {
    setSuperClass(superClass);
    super.close(interfaces, null, fields_H, methods, methods_H);
  }

  /**
   * Returns true if initialization pass is finished for this class
   */
  public boolean initializerChecked() {
    return initializerChecked;
  }

  /**
   * Sets initialization pass to be finished
   */
  public void setInitializerChecked() {
    initializerChecked = true;
  }

  // ----------------------------------------------------------------------
  // ACCESSORS
  // ----------------------------------------------------------------------

  /**
   * add synthetic parameters to method def
   */
  public CType[] genConstructorArray(CType[] params) {
    boolean		hasOuterThis = isInnerClass() && !isStatic() && hasOuterThis();
    int			size = params.length + (hasOuterThis ? 1 : 0) + (outers == null ? 0 : outers.size());
    if (params.length != size) {
      Enumeration	enum = outers == null ? null : outers.keys();
      CType[]		ret = new CType[size];
      int			pos = 0;
      
      for (; pos < params.length; pos++) {
	ret[pos] = params[pos];
      }
      
      if (hasOuterThis) {
	ret[pos++] = getOwner().getType();
      }
      if (enum != null) {
	while (enum.hasMoreElements()) {
	  JLocalVariable	var = (JLocalVariable)enum.nextElement();
	  ret[pos++] = var.getType();
	}
      }

      return ret;
    }

    return params;
  }

  /**
   * add synthetic parameters to method call
   */
  public void genSyntheticParams(CodeSequence code) {
    if (isInnerClass() && !isStatic() && hasOuterThis()) {
      code.plantLoadThis();
    }
    if (outers != null) {
      Enumeration	enum = outers.keys();
      while (enum.hasMoreElements()) {
	JLocalVariable	var = (JLocalVariable)enum.nextElement();

	new JLocalVariableExpression(TokenReference.NO_REF, var).genCode(code);
      }
    }
  }

  /**
   * add synthetic parameters to method call
   */
  public void genSyntheticParamsFromExplicitSuper(JExpression[] params, CodeSequence code) {
    if (isInnerClass() && !isStatic() && hasOuterThis()) {
      JGeneratedLocalVariable var = new JGeneratedLocalVariable(null, 0, getOwner().getType(), JAV_OUTER_THIS, null);

      var.setPosition(params.length + 1);
      new JLocalVariableExpression(TokenReference.NO_REF, var).genCode(code);
    }
  }

  /**
   * Gets the code to access outer local vars
   */
  public JExpression getOuterLocalAccess(TokenReference ref, JLocalVariable var, final CMethod constructor) {
    String		name;
    CSourceField	field;

    if (outers == null) {
      outers = new Hashtable();
    }
    if ((name = (String)outers.get(var)) == null) {
      outers.put(var, (name = var.getIdent() + "_$")); // local vars are uniq
      field = new CSourceField(this, 0, name, var.getType(), false);
      addField(field);
      countSyntheticsFields++;
    } else {
      field = (CSourceField)getField(name);
    }

    if (constructor != null) {
      final	CSourceField	ffield = field;
      JGeneratedLocalVariable local = new JGeneratedLocalVariable(null, 0, var.getType(), var.getIdent(), null) {
	  /**
	   * @return the local index in context variable table
	   */
	  public int getPosition() {
	    return 1 /*this*/ + constructor.getParameters().length + (hasOuterThis() ? 1 : 0) + ffield.getPosition() - (getFieldCount() - countSyntheticsFields);
	  }
	};

      return new JCheckedExpression(ref, new JLocalVariableExpression(ref, local));
    } else {
      return new JClassFieldExpression(ref, new JThisExpression(ref, getCClass()), name);
    }
  }

  /**
   * add synthetic parameters to method def
   */
  public void genInit(CodeSequence code, int countLocals) {
    countLocals++; // this

    if (isInnerClass() && !isStatic() && (hasOuterThis() || (outers != null && outers.size() > 0))) {
      JGeneratedLocalVariable var;
      if (hasOuterThis()) {
	var = new JGeneratedLocalVariable(null, 0, getOwner().getType(), JAV_OUTER_THIS, null); // $$$
	var.setPosition(countLocals);
	countLocals += var.getType().getSize();
	code.plantLoadThis();
	var.genLoad(code);
	code.plantInstruction(new FieldRefInstruction(opc_putfield,
						      getQualifiedName(),
						      JAV_OUTER_THIS,
						      getOwner().getType().getSignature()));
      }
      if (outers != null) {
	Enumeration	enum = outers.keys();
	while (enum.hasMoreElements()) {
	  JLocalVariable	ovar = (JLocalVariable)enum.nextElement();
	  var = new JGeneratedLocalVariable(null, 0, ovar.getType(), ovar.getIdent(), null);
	  var.setPosition(countLocals);
	  countLocals += var.getType().getSize();
	  code.plantLoadThis();
	  var.genLoad(code);
	  code.plantInstruction(new FieldRefInstruction(opc_putfield,
							getQualifiedName(),
							var.getIdent() + "_$",
							var.getType().getSignature()));
	}
      }
    }
  }

  // ----------------------------------------------------------------------
  // CODE GENERATION
  // ----------------------------------------------------------------------

  /**
   * Collect and generate class files
   */
  public void genCode(String destination)
    throws IOException, ClassFileFormatException
  {
    genClassFile(destination);
  }

  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  private	Hashtable		outers;
  private	int			countSyntheticsFields;
  private	boolean			initializerChecked;
}
