/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: JBooleanLiteral.java,v 1.2 2002/03/08 21:54:24 mrobilla Exp $
 */

package at.dms.kjc;

import at.dms.classfile.PushLiteralInstruction;
import at.dms.compiler.TokenReference;

/**
 * Root class for all expressions
 */
public class JBooleanLiteral extends JLiteral {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * Construct a node in the parsing tree
   * @param	where		the line of this node in the source code
   * @param	image		the string representation of this literal
   */
  public JBooleanLiteral(TokenReference where, boolean value) {
    super(where);
    this.value = value;
  }

  // ----------------------------------------------------------------------
  // ACCESSORS
  // ----------------------------------------------------------------------

  /**
   * Compute the type of this expression (called after parsing)
   * @return the type of this expression
   */
  public CType getType() {
    return CStdType.Boolean;
  }

  /**
   * get a Number that represents the value of this literal
   * Do not call before checkBody
   */
  public boolean booleanValue() {
    return value;
  }

  /**
   * Returns the literal value of an expression
   */
  public boolean isDefault() {
    return !booleanValue();
  }

  /**
   * get a Number that represents the value of this literal
   * Do not call before checkBody
   */
  public Object getValue() {
    return new Integer(value ? 1 : 0);
  }

  /**
   * set the value of this literal
   * @param	newValue		the new value
   */
  public void setValue(boolean newValue) {
    value = newValue;
  }

  // ----------------------------------------------------------------------
  // CODE CHECKING
  // ----------------------------------------------------------------------

  /**
   * Check expression and evaluate and alter context
   * @param context the actual context of analyse
   * @return  a pure java expression including promote node
   * @exception	PositionedError		if the check fails
   */
  public JExpression checkExpression(CExpressionContext context) {
    return this;
  }

  // ----------------------------------------------------------------------
  // CODE GENERATION
  // ----------------------------------------------------------------------

  /**
   * Accepts the specified visitor
   * @param	p		the visitor
   */
  public void accept(KjcVisitor p) {
    p.visitBooleanLiteral(value);
  }

  /**
   * Generates a sequence of bytescodes
   * @param	code		the code list
   */
  public void genCode(CodeSequence code) {
    code.setLineNumber(getTokenReference().getLine());

    code.plantInstruction(new PushLiteralInstruction(value ? 1 : 0));
  }

  /**
   * Generates a sequence of bytescodes to branch on a label
   * This method helps to handle heavy optimizables conditions
   * @param	code		the code list
   */
  public void genBranch(boolean cond, CodeSequence code, CodeLabel label) {
    code.setLineNumber(getTokenReference().getLine());

    if (value == cond) {
      code.plantLabelRef(opc_goto, label);
    }
  }


  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  private boolean		value;
}
