/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: JConditionalAndExpression.java,v 1.2 2002/03/08 21:54:27 mrobilla Exp $
 */

package at.dms.kjc;

import at.dms.compiler.PositionedError;
import at.dms.compiler.TokenReference;

/**
 * This class implements the conditional and operation
 */
public class JConditionalAndExpression extends JBinaryExpression {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * Construct a node in the parsing tree
   * This method is directly called by the parser
   * @param	where		the line of this node in the source code
   * @param	left		the left operand
   * @param	right		the right operand
   */
  public JConditionalAndExpression(TokenReference where,
				   JExpression left,
				   JExpression right) {
    super(where, left, right);
  }

  // ----------------------------------------------------------------------
  // CODE CHECKING
  // ----------------------------------------------------------------------

  /**
   * Check expression and evaluate and alter context
   * @param context the actual context of analyse
   * @return  a pure java expression including promote node
   * @exception	PositionedError		if the check fails
   * @exception	UnpositionedError	this error will be positioned soon
   */
  public JExpression checkExpression(CExpressionContext context) throws PositionedError {
    left = left.checkExpression(context);
    right = right.checkExpression(context);

    CType	leftType = left.getType();
    CType	rightType = right.getType();

    check(context, leftType == CStdType.Boolean &&  rightType == CStdType.Boolean,
	  KjcMessages.AND_BADTYPE, leftType, rightType);

    type = CStdType.Boolean;

    if (left.isConstant() || right.isConstant()) {
      if (left.isConstant() && right.isConstant()) {
	JBooleanLiteral	leftLit = left.getBooleanLiteral();
	JBooleanLiteral	rightLit = right.getBooleanLiteral();

	left = new JBooleanLiteral(getTokenReference(), leftLit.booleanValue() && rightLit.booleanValue());
      } else if (left.isConstant()) {
	JBooleanLiteral	leftLit = left.getBooleanLiteral();
	if (leftLit.booleanValue()) {
	  left = right;
	}
      } else {
	JBooleanLiteral	rightLit = right.getBooleanLiteral();
	if (!rightLit.booleanValue()) {
	  left = right;
	}
      }

      return left;
    }

    return this;
  }

  // ----------------------------------------------------------------------
  // CODE GENERATION
  // ----------------------------------------------------------------------

  /**
   * Accepts the specified visitor
   * @param	p		the visitor
   */
  public void accept(KjcVisitor p) {
    p.visitBinaryExpression(this, "&&", left, right);
  }

  /**
   * Generates a sequence of bytescodes
   * @param	code		the code list
   */
  public void genCode(CodeSequence code) {
    genBooleanResultCode(code);
  }

  /**
   * Optimize a bi-conditional expression
   */
  protected void genBranch(JExpression left,
			   JExpression right,
			   boolean cond,
			   CodeSequence code,
			   CodeLabel label)
  {
    if (cond) {
      CodeLabel		skip = new CodeLabel();
      left.genBranch(false, code, skip);
      right.genBranch(true, code, label);
      code.plantLabel(skip);
    } else {
      left.genBranch(false, code, label);
      right.genBranch(false, code, label);
    }
  }
}
