/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: JInitializerDeclaration.java,v 1.2 2002/03/08 21:54:30 mrobilla Exp $
 */

package at.dms.kjc;

import at.dms.compiler.PositionedError;
import at.dms.compiler.TokenReference;

/**
 * This class represents a java class in the syntax tree
 */
public class JInitializerDeclaration extends JMethodDeclaration {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * Construct a node in the parsing tree
   * This method is directly called by the parser
   * @param	where		the line of this node in the source code
   * @param	parent		parent in which this methodclass is built
   */
  public JInitializerDeclaration(TokenReference where,
				 JBlock body,
				 boolean isStatic,
				 boolean isDummy) {
    super(where,
	  ACC_PRIVATE + (isStatic ? ACC_STATIC : 0),
	  CStdType.Void,
	  (isStatic ? JAV_STATIC_INIT : JAV_INIT),
	  JFormalParameter.EMPTY,
	  CClassType.EMPTY,
	  body,
	  null,
	  null);

    this.isDummy = isDummy;
  }

  // ----------------------------------------------------------------------
  // ACCESSORS
  // ----------------------------------------------------------------------

  /**
   * Return true if this initialiser declaration is used only to check code
   * and that it should not generate code
   */
  public boolean isDummy() {
    return isDummy;
  }

  // ----------------------------------------------------------------------
  // CODE CHECKING
  // ----------------------------------------------------------------------

  /**
   * createContext
   * @param	parent		the parent context or null
   * @return	returns a CClassContext that represents this context
   * @exception	UnpositionedError	Error catched as soon as possible
   */
  public CMethodContext createContext(CClassContext parent) {
    return new CInitializerContext(parent, getMethod());
  }

  /**
   * Check expression and evaluate and alter context
   * @param	context		the actual context of analyse
   * @return	a pure java expression including promote node
   * @exception PositionedError Error catched as soon as possible
   */
  public void checkStatement(CClassContext context) {
    // I've made it !
    // nothing to check.
  }

  protected void checkAccess(CClassContext context, JMethodDeclaration superMethod) {
    // nothing to do here
  }

  protected void checkBody(CClassContext context) {
    // nothing to do here
  }

  protected void checkOverriding(CClassContext context, CMethod superMethod) throws PositionedError {
    // nothing to do here
  }
  
  /**
   * Check expression and evaluate and alter context
   * @param	context		the actual context of analyse
   * @return	a pure java expression including promote node
   * @exception	PositionedError		Error catched as soon as possible
   * @exception	UnpositionedError	Error catched as soon as possible
   */
  public void checkInitializer(CClassContext context) throws PositionedError {
    if (!getMethod().isStatic() && !isDummy()) {
      context.addInitializer();
    }
    super.checkStatement(context); // we want to do that at check intitializers time
  }

  // ----------------------------------------------------------------------
  // CODE GENERATION
  // ----------------------------------------------------------------------

  /**
   * Accepts the specified visitor
   * @param	p		the visitor
   */
  public void accept(KjcVisitor p) {
    // Don t print initializers
  }

  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  private boolean	isDummy;
}
