/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: JInstanceofExpression.java,v 1.2 2002/03/08 21:54:31 mrobilla Exp $
 */

package at.dms.kjc;

import at.dms.classfile.ClassRefInstruction;
import at.dms.compiler.CWarning;
import at.dms.compiler.PositionedError;
import at.dms.compiler.TokenReference;
import at.dms.compiler.UnpositionedError;

/**
 * This class represents a instanceof expression.
 */
public class JInstanceofExpression extends JExpression {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * Construct a node in the parsing tree
   * This method is directly called by the parser
   *
   * @param	where	the line of this node in the source code
   * @param	expr	the expression to be casted
   * @param	dest	the type to test for
   */
  public JInstanceofExpression(TokenReference where, JExpression expr, CType dest) {
    super(where);
    this.expr = expr;
    this.dest = dest;
  }

  // ----------------------------------------------------------------------
  // ACCESSORS
  // ----------------------------------------------------------------------

  /**
   * Compute the type of this expression.
   *
   * @return the type of this expression
   */
  public CType getType() {
    return CStdType.Boolean;
  }

  // ----------------------------------------------------------------------
  // CODE CHECKING
  // ----------------------------------------------------------------------

  /**
   * Check expression and evaluate and alter context
   * @param context the actual context of analyse
   * @return  a pure java expression including promote node
   * @exception	PositionedError		if the check fails
   * @exception	UnpositionedError	this error will be positioned soon
   */
  public JExpression checkExpression(CExpressionContext context) throws PositionedError {
    expr = expr.checkExpression(context);
    try {
      dest.checkType(context);
    } catch (UnpositionedError e) {
      throw e.addPosition(getTokenReference());
    }
    check(context, expr.getType().isReference(),
	  KjcMessages.INSTANCEOF_BADTYPE, expr.getType(), dest);

    check(context, dest.isCastableTo(expr.getType()),
	  KjcMessages.INSTANCEOF_BADTYPE, expr.getType(), dest);

    if (expr.getType().implicitlyCastTo(dest)) {
      context.reportTrouble(new CWarning(getTokenReference(), KjcMessages.UNNECESSARY_INSTANCEOF, null));
    }

    return this;
  }

  // ----------------------------------------------------------------------
  // CODE GENERATION
  // ----------------------------------------------------------------------

  /**
   * Accepts the specified visitor
   * @param	p		the visitor
   */
  public void accept(KjcVisitor p) {
    p.visitInstanceofExpression(this, expr, dest);
  }

  /**
   * Generates a sequence of bytescodes
   * @param	code		the code list
   */
  public void genCode(CodeSequence code) {
    code.setLineNumber(getTokenReference().getLine());

    expr.genCode(code);

    code.plantInstruction(new ClassRefInstruction(opc_instanceof,
						  ((CClassType)dest).getQualifiedName()));
  }

  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  private JExpression		expr;
  private CType			dest;
}
