/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: JThrowStatement.java,v 1.2 2002/03/08 21:54:38 mrobilla Exp $
 */

package at.dms.kjc;

import at.dms.classfile.NoArgInstruction;
import at.dms.compiler.PositionedError;
import at.dms.compiler.TokenReference;
import at.dms.compiler.JavaStyleComment;

/**
 * JLS 14.16: Throw Statement
 *
 * A throw statement causes an exception to be thrown.
 */
public class JThrowStatement extends JStatement
{

    // ----------------------------------------------------------------------
    // CONSTRUCTORS
    // ----------------------------------------------------------------------
    
    /**
     * Construct a node in the parsing tree
     * @param	where		the line of this node in the source code
     * @param	expr		the expression to throw.
     */
    public JThrowStatement(TokenReference where, JExpression expr, JavaStyleComment[] comments)
    {
	super(where, comments);
	this.expr = expr;
    }

    // ----------------------------------------------------------------------
    // CODE CHECKING
    // ----------------------------------------------------------------------
    
    /**
     * Check statement.
     *
     * @param	context		the actual context of analyse
     * @exception	PositionedError		if the check fails
     * @exception	UnpositionedError	this error will be positioned soon
     */
    public void checkStatement(CBodyContext context) throws PositionedError
    {
	expr = expr.checkExpression(context.createExpressionContext());
	check(context, expr.getType().isReference() &&
	      expr.isAssignableTo(CStdType.Throwable),
	      KjcMessages.THROW_BADTYPE, expr.getType());

	if (expr.getType().isCheckedException()) {
	    context.addThrowable(new CThrowableInfo((CClassType)expr.getType(),
						    this,
						    context.makeSnapshot()));
	}
	context.setReachable(false);
    }

  // ----------------------------------------------------------------------
  // CODE GENERATION
  // ----------------------------------------------------------------------

  /**
   * Accepts the specified visitor
   * @param	p		the visitor
   */
  public void accept(KjcVisitor p) {
    super.accept(p);
    p.visitThrowStatement(this, expr);
  }

  /**
   * Generates a sequence of bytescodes
   * @param	code		the code list
   */
  public void genCode(CodeSequence code) {
    code.setLineNumber(getTokenReference().getLine());

    expr.genCode(code);
    code.plantInstruction(new NoArgInstruction(opc_athrow));
  }

    // ----------------------------------------------------------------------
    // PRIVATE DATA MEMBERS
    // ----------------------------------------------------------------------

    private JExpression		expr;
}
