/* Jex - A tool for analyzing exception flow in Java programs
 *
 *  Copyright (C) 2001 Martin P. Robillard (mrobilla@cs.ubc.ca)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package ca.ubc.cs.jex;

import at.dms.kjc.CClass;
import java.io.PrintWriter;
import java.util.Iterator;

/** Represents a Jex File for an anonymous class
  * @version $Revision: 1.2 $ $Date: 2002/03/08 21:56:44 $
  */
public class AnonymousJexFile extends JexFile
{
    /** Initializes the AnonymousJexFile with the
      * corresponding class.
      * @param pClass The class that the Jex file represents.
      */
    public AnonymousJexFile( CClass pClass )
    { super( pClass ); }

    /** Returns true because the Jex file is anonymous.
      * @return Always returns true;
      */
    public boolean isAnonymous()
    { return true; }

    /** Writes out the anonymous Jex file to a PrintWriter
      * @param pOut The PrintWriter to write the Jex file to.
      */ 
    public void write( PrintWriter pOut )
    {
	String lType = aClass.getSuperClass().getQualifiedName().replace('/','.');
	if( aClass.interfaces != null )
	{
	    if( aClass.interfaces.length > 0 )
	    {
		for( int i = 0; i < aClass.interfaces.length; i++ )
		    lType += "," + aClass.interfaces[i].getQualifiedName().replace('/','.');
	    }
	}
	pOut.println( "*anonymous: " + lType + ";" );
	super.write( pOut );
    }

    /** Copies method headers only */
    public JexFile copy()
    {
	AnonymousJexFile lReturn = new AnonymousJexFile( aClass );
	Iterator lMethods = aMethods.values().iterator();
	while( lMethods.hasNext() )
	{
	    JexMethod lNewMethod = ((JexMethod)lMethods.next()).copy();
	    lReturn.addJexMethod( lNewMethod );
	}
	return lReturn;
    }
}
