/* Jex - A tool for analyzing exception flow in Java programs
 *
 *  Copyright (C) 2001 Martin P. Robillard (mrobilla@cs.ubc.ca)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package ca.ubc.cs.jex;

import java.io.PrintWriter;

/** Represents an exception that is explicitly raised
  * (i.e., raised using the throw keyword.
  * @version $Revision: 1.2 $ $Date: 2002/03/08 21:56:45 $
  */
public class ExceptionThrow extends SingleExceptionElement
{
    /** Initializes the descriptor with the class type
     * of the exception raised.
     * @param pType The class type of the exception raised.
     */
    public ExceptionThrow( Class pType )
    { super( pType ); }

    /** Writes out the exception descriptor to a PrintWriter
      * @param pFile The PrintWriter to write the descriptor to.
      * @param pPrefix The indentation.
      */
    public void write( PrintWriter pFile, String pPrefix )
    {
	pFile.println( pPrefix + "throws " + getType().getName() + ";" );
    }
 
    /** Returns if pContainer is equal to this exception descriptor.
      * Two ExceptionThrow a equal if their class types are equal.
      */
    public boolean equals( Object pContainer )
    {
	boolean lReturn = false;

	if( pContainer instanceof ExceptionThrow )
	{
	    if( ((ExceptionThrow)pContainer).getType().equals( getType() ) ) 
		lReturn = true;
	}
	return lReturn;
    }
}
