/* Jex - A tool for analyzing exception flow in Java programs
 *
 *  Copyright (C) 2001 Martin P. Robillard (mrobilla@cs.ubc.ca)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package ca.ubc.cs.jex;

import at.dms.kjc.CClass;
import at.dms.kjc.CClassType;
import at.dms.kjc.JMethodDeclaration;
import java.io.PrintWriter;
import java.util.Set;

/** Represents a Jex File
  * @version $Revision: 1.2 $ $Date: 2002/03/08 21:56:46 $
  */
public class JexMethod
{
    private String aSignature;
    private String aThrows = "";
    private ExceptionBlock aElements;

    public JexMethod( String pSignature, CClassType[] pExs )
    { 
	aSignature = pSignature;
	aElements = new ExceptionBlock();
	if( pExs  != null )
	{
	    if( pExs.length > 0 )
	    {
		aThrows += "throws ";
		for( int i = 0; i < pExs.length -1; i++ )
		    aThrows += pExs[i].getQualifiedName().replace('/','.') + ", ";
		aThrows += pExs[pExs.length - 1].getQualifiedName().replace('/','.');
	    }
	}
    }

    public JexMethod( String pSignature, String pThrows )
    {
	aSignature = pSignature;
	aThrows = pThrows;
	aElements = new ExceptionBlock();
    }

    /** Copy constructors, does not copy exceptions structure, just header */
    public JexMethod copy()
    {
    	return new JexMethod( aSignature, aThrows );
    }

    public void addElement( ExceptionElement pElement )
    {
	aElements.addElement( pElement );
    }

    public String getSignature()
    { return aSignature; }

    public ExceptionBlock getExceptionBlock()
    { return aElements; }

    public void setExceptionBlock( ExceptionBlock pBlock )
    { aElements = pBlock; }

    public Set getExceptions()
    {
	return aElements.getExceptions();
    }

    public void write( PrintWriter pOut, ExceptionBlock pInit )
    {
	pOut.println( aSignature + " " + aThrows );
	if( aSignature.startsWith( "<init>" ))
	    aElements.write( pOut, "", pInit );
	else
	    aElements.write( pOut, "" );
    }

    public int getCheckSum()
    {
	return aElements.getCheckSum();
    }

    // For testing
    public void dump()
    {
	System.out.println( aSignature );
    }

    //public void clear()
    //{
    //	aElements.clear();
    //}
}
