/* Jex - A tool for analyzing exception flow in Java programs
 *
 *  Copyright (C) 2001 Martin P. Robillard (mrobilla@cs.ubc.ca)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package ca.ubc.cs.jex.file;

import java.io.File;
import java.io.IOException;
import java.util.Vector;

/** Represents a path to Jex files
  * @version $Revision: 1.2 $ $Date: 2002/03/08 21:55:50 $
  */
public class JexPath
{
    private String aPath;

    public JexPath( String pPath )
    { aPath = pPath; }

    public String getPath()
    { return aPath; }

    public String makeJexDirectoryName( String pPackageName )
    {
	return aPath + File.separator + pPackageName.replace( '.', File.separatorChar );
    }

    public String makeJexFileName( String pPackageName, String pClassName )
    {
	return aPath + File.separator + pPackageName.replace( '.', File.separatorChar ) +
	    File.separatorChar + pClassName + ".jex";
    }

    public String makeJexFileName( String pClassName )
    {
	return aPath + File.separator + pClassName.replace( '.', File.separatorChar ) + ".jex";
    }

    public String[] getAnonymousJexFiles( String pPackageName ) throws IOException
    {
	String lDirName = makeJexDirectoryName( pPackageName );
	File lPackageFile = new File( lDirName );
	
	if( lPackageFile.exists() != true )
	    throw new IOException( "Directory not found: " + lDirName );

	String[] lFiles = lPackageFile.list();
	Vector lValidFiles = new Vector();
	
	for( int i = 0; i < lFiles.length; i++ )
	{
	    if( ((new File( lDirName + File.separator + lFiles[i]).isFile()) &&
		 ( lFiles[i].endsWith( ".jex" )) &&
		 ( lFiles[i].indexOf( "$$" ) != -1 )))
	    {
		lValidFiles.addElement( lFiles[i].substring( 0, lFiles[i].length() -4 ));
	    }
		else
		;// No an anonymous jex file, don't add.
	}
	
	String[] lReturn = null;
	if( lValidFiles.size() > 0 )
	{
	    lReturn = new String[ lValidFiles.size() ];
	    for( int i = 0; i < lValidFiles.size(); i++ )
		lReturn[i] = (String)lValidFiles.elementAt(i);
	}
	else lReturn = new String[0];
	return lReturn;
    }

    // For testing purposes
    public static void main( String[] args ) throws IOException
    {
	JexPath lPath = new JexPath( args[0] );
	System.out.println( lPath.getPath() );
	System.out.println( lPath.makeJexDirectoryName( args[1] ));
	System.out.println( lPath.makeJexFileName( args[1], args[2] ));
	String[] lClasses = lPath.getAnonymousJexFiles( args[1] );
	for( int i = 0; i < lClasses.length; i++ )
	    System.out.println( lClasses[i] );
    }
}
