/* Jex - A tool for analyzing exception flow in Java programs
 *
 *  Copyright (C) 2001 Martin P. Robillard (mrobilla@cs.ubc.ca)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package ca.ubc.cs.jex.util;

import java.io.File;
import java.io.IOException;
import java.io.FilenameFilter;
import java.util.Vector;
import java.util.StringTokenizer;

/** A DirectoryPackageAccessor can accesses a package from a directory.
  * @author $Author: mrobilla $
  * @version $Revision: 1.2 $ $Date: 2002/03/08 21:56:21 $
  */
public class DirectoryPackageAccessor extends PackageAccessor
{
    public DirectoryPackageAccessor( File pPath )
    {
	super( pPath );
    }

    /** Returns true if the argument is a valid directory
      * @param pFile File to check if we can match to.
      * @return true if the argument is a valid directory
      */
    public static boolean match( File pFile )
    {
	if( (pFile.exists()) && (pFile.isDirectory()) )
	    return true;
	else
	    return false;
    }

    /** Returns the names of the classes contained in the package specified as argument,
      * @param pPackageName Fully qualified package name
      * @return An array of strings containing the classes in the package.  Null if the
      * package cannot be found in the current structure.
      * @exception IOException if it is impossible to open the
      * specified file.
      */
    public String[] getClasses( String pPackageName ) throws IOException
    {
	String lPath = createAbsoluteFileName( aAccessor, 
					       pPackageName.replace( '.', File.separatorChar ));
	File lPackageFile = new File( lPath );
	
	if( lPackageFile.exists() != true )
	    return null;

	String[] lClasses = lPackageFile.list();
	Vector lValidClasses = new Vector();
	
	for( int i = 0; i < lClasses.length; i++ )
	{
	    if( ((new File( lPath + File.separator + lClasses[i])).isFile()) &&
		( lClasses[i].endsWith( FileTypesI.kClass ) ))
	    {
		lValidClasses.addElement( lClasses[i].substring( 0, lClasses[i].length() - 6 ) );
	    }
	    else
		;// Not a .class file, don't add.
	}

	String[] lReturn = null;
	if( lValidClasses.size() > 0 )
	{
	    lReturn = new String[ lValidClasses.size() ];
	    for( int i = 0; i < lValidClasses.size(); i++ )
		lReturn[i] = (String)lValidClasses.elementAt(i);
	}
	return lReturn;
    }
}

class ClassFilenameFilter implements FilenameFilter
{
    public boolean accept( File pDir, String pName )
    {
	return pName.endsWith( FileTypesI.kClass  );
    }
}

