/* Jex - A tool for analyzing exception flow in Java programs
 *
 *  Copyright (C) 2001 Martin P. Robillard (mrobilla@cs.ubc.ca)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package ca.ubc.cs.jex.util;

import java.lang.reflect.Method;
import java.util.Vector;

/** A MethodRelationship keeps the links between a method
  * and all the methods that can override or implement it.
  * @author $Author: mrobilla $
  * @version $Revision: 1.2 $ $Date: 2002/03/08 21:56:22 $
  */
public interface MethodRelationships
{
    /** Adds all the methods of all the classes in the
      * package to the relationship database.
      * @param pPackage The fully qualified name of the
      * package to add.
      * @exception PackageNotFoundException if the
      * package cannot be loaded.
      */
    public void addPackage( String pPackage ) throws MethodRelationshipsException;

    /** Returns all the methods that directly or indirectly
      * implement or override the argument method.
      * @param pMethod The parent method.
      * @return A vector of method ovverriding or implementing
      * the argument method.
      * @exception MethodRelationshipsException if the argument
      * method cannot be found in the database.
      */
    public Vector getImplementations( Method pMethod ) throws MethodRelationshipsException;
}	    
